package uk.ac.shef.is.imageClef.trecResult;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/** Class parses a normal trec-format
 * result file.
 * @author Thomas Arni
 *
 */
public class TrecResultParser {
		
	public static TrecResultsContainer parse(File fileToParse){
		TrecResultsContainer tr = new TrecResultsContainer();	
		try {
			BufferedReader input = new BufferedReader(new InputStreamReader(
					new FileInputStream(fileToParse)));
			try {
				tr.setFilePath(fileToParse.getAbsolutePath());
				String line = null; //not declared within while loop
				while ((line = input.readLine()) != null) {
					TrecResultEntryContainer tre =parseLine(line,fileToParse);
					if(tr.getRunName() == null){
						tr.setRunName(tre.getRunName());
					}
					if(tre != null){
						TrecTopicResultsContainer ttr= tr.getTopicResults(tre.getTopicNr()); //get existing TrecTopicResultsContainer or create new one 
						ttr.addTrecResultEntry(tre);
					}
				}
			} finally {
				input.close();
			}
		} catch (IOException ex) {
			System.err.println("Error during file reading : " + fileToParse.getAbsolutePath());
		}	
		return tr;
	}
	
	/** Return a TrecTopicResultsContainer with the content of the TREC-style result for a
	 * specif topic
	 * @param fileToParse TREC-style result file to parse
	 * @param topicNr Requested topic number
	 * @return TrecTopicResultsContainer
	 */
	public static TrecTopicResultsContainer parse(File fileToParse, String topicNr){
		TrecTopicResultsContainer ttr = new TrecTopicResultsContainer(topicNr);	
		try {
			BufferedReader input = new BufferedReader(new InputStreamReader(
					new FileInputStream(fileToParse)));
			try {
				ttr.setFilePath(fileToParse.getAbsolutePath());
				String line = null; //not declared within while loop
				while ((line = input.readLine()) != null) { // parse each line					
					TrecResultEntryContainer tre = parseLine(line,fileToParse);
					if(tre != null && tre.getTopicNr().equals(topicNr)){
//						System.out.println(">"+line);						
						ttr.addTrecResultEntry(tre);
					}
				}
			} finally {
				input.close();
			}
		} catch (IOException ex) {
			System.err.println("Error during file reading : " + fileToParse.getAbsolutePath());
		}	
		return ttr;
	}
	
	/** Parse one line from the result file
	 * @param line current line of the trec-style result file
	 * @return TrecResultEntry with all relevant info (topicNr, docNr, rank)
	 */
	public static TrecResultEntryContainer parseLine(String line, File fileToCheck){
		line = line.trim();
		line = line.replaceAll("\t", " "); //replace tab with space
		line = line.replaceAll("[ ]{1,10} ", " "); //replace multiple space by just one space
		String myPattern = "([0-9]{1,2}) 1 ([0-9]{1,2}/[0-9]{1,5}) ([0-9]{1,4}) (.*) (.*)";
		Pattern pattern = Pattern.compile(myPattern);
	    Matcher matcher = pattern.matcher(line);

	    if(matcher.find()){
	    	String topicNr = matcher.group(1);
	    	String docNr = matcher.group(2);
	    	String rank = matcher.group(3);
	    	String score = matcher.group(4);
	    	String runName = matcher.group(5);
	    	boolean correctTopicNr = false;
	    	boolean correctDocNr = false;
	    	boolean correctRank = false;
	    	boolean correctScore = false;
	    	boolean correctRunName = false;
	    	
		    if(topicNr != null && topicNr.length() > 0){
		    	correctTopicNr = true;
		    }
		    else System.err.println("Could not get topicNr for line: " + line + " in file " + fileToCheck.getAbsoluteFile());
		    
		    if(docNr != null && docNr.length() > 0){
		    	correctDocNr = true;
		    }
		    else System.err.println("Could not get docNr for line: "  + line + " in file " + fileToCheck.getAbsoluteFile());
		    
		    if(rank != null && rank.length() > 0){
		    	correctRank = true;
		    }
		    else System.err.println("Could not get rank for line: " + line + " in file " + fileToCheck.getAbsoluteFile());	
			    
		    if(score != null && score.length() > 0){
		    	correctScore = true;
		    }
		    else System.err.println("Could not get score for line: " + line + " in file " + fileToCheck.getAbsoluteFile());	
		    
		    if(runName != null && runName.length() > 0){
		    	correctRunName = true;
		    }
		    else System.err.println("Could not get a run identifier for line: " + line + " in file " + fileToCheck.getAbsoluteFile());	
			    
		    if(correctDocNr && correctRank && correctScore && correctTopicNr && correctRunName){
		    	return new TrecResultEntryContainer(topicNr,docNr,rank,score,runName);
		    }
		    else System.err.println("Could not create TrecResultEntry on line: " + line + " in file " + fileToCheck.getAbsoluteFile());
	    }
	    else{
	    	System.err.println("Could not find required regex pattern on line >"+ line + "< in file " + fileToCheck.getAbsoluteFile());
	    	System.err.println("Please follow the format guidelines!");
	    }
	    return null;
	}
}
